#!/usr/bin/env python
# -*- coding: utf-8 -*-
from datetime import datetime

from tvm.commands import base

from tvm.helpers.play import PlayHelper

from tvm.shared.io import err, info, info2, ok, warn
from tvm.shared.io import print_matched_status

from tvm.shared.dates import time_now_str, time_to_str, datetime_to_time

from tvm.shared.sources.video import VideoSource
from tvm.shared.tvstatistics import TVStatistics


class PlayCommand(base.BaseCommand):
    def __init__(self, **arguments):
        super(self.__class__, self).__init__(**dict(arguments, **{'_ignore_errors': True}))

        self.helper = PlayHelper(self.config, self.collection)
        self._extact_args(arguments)
        self._exec()

    def _extact_args(self, arguments):
        # Join by '_' and not ' ' because internally, words are '_' delimited.
        self._search_query = '_'.join(arguments.get('<title>', []))
        self._no_open = arguments.get('--no-open', False)
        self._undo = arguments.get('--undo', False)
        self._random = arguments.get('--random', False)

    def _exec(self):
        if self._undo:
            removed_episode = self.helper.undo_last_played()
            if not removed_episode:
                return err("Couldn't open shows_viewed.csv to remove most recently viewed.")
            return ok("'%s' --undo was successful." % removed_episode)

        if not self._search_query:
            return warn('The title, "%s" was invalid. tvm -h for more help.' % self._search_query)

        matched = self.collection.find_matching_titles(self._search_query)
        print_matched_status(matched)

        if self._random:
            return self.helper.simulate_random_selection(matched)

        next_to_watch = self.helper.next_to_watch(matched)
        if not next_to_watch:
            info('No new episodes for "%s".' % matched)
        else:
            if self._no_open:
                info('Not opening the TV show "%s".' % ','.join(next_to_watch))
                return self.helper.update_records_with(next_to_watch)

            info('Opening the episode, "%s".' % ','.join(next_to_watch))
            if not self.helper.play_episode(next_to_watch):
                err('Cannot open episode, "%s".' % ','.join(next_to_watch))
                info2('Try the --no-open option to record and not play the episode.')
            else:
                self.helper.update_records_with(next_to_watch)

                # Display the duration of the episode.
                duration = VideoSource.duration(self.helper._build_episode_path(next_to_watch))
                info2('%s -- %s (duration: %s minutes).' % (
                    time_now_str(),
                    time_to_str(datetime_to_time(datetime.now() + duration)),
                    duration.seconds / 60)
                )

                # Display the percentage of viewed episodes.
                statistics = TVStatistics(self.collection)
                percentage, recorded, viewed = statistics.percentage_viewed(matched)

                info2('%d%% (%d/%d) total views.' % (percentage, viewed, recorded))
