#!/usr/bin/env python
# -*- coding: utf-8 -*-
from __future__ import print_function

import os
import json
from datetime import datetime

from tvm.helpers import base
from tvm.shared.io import info, info2
from tvm.shared.io import open_with_default
from tvm.shared.notification import Notifier


class CheckHelper(base.BaseHelper):
    def __init__(self, config, collection):
        super(self.__class__, self).__init__(config, collection)

    def _notify_message(self, new_episodes):
        string_options = (len(new_episodes), ', '.join([t['tracked_title'] for t in new_episodes]))
        if len(new_episodes) == 1:
            return 'There\'s a new episode of %s to download.' % new_episodes[0]['tracked_title']
        return '%d new episodes to download: %s.' % string_options

    def notify(self, rss, cache_notification=True):
        """ Set `cache_notification=False` to not cache notification. """
        notification = self.get_previous_notification()

        # TODO: The check to notify can be improved.
        #   - Even if the notification matches rss.new_tracked_and_downloadable, if it exceeds
        #     12hrs then it should notify anyway.
        #   - If I get a notification for x, y and z. I then download z. The next
        #     notification will be x and y only. This means a notification message will appear.
        #
        #     Should this be accounted for or should this be a "feature"? The "feature" being
        #     that there's more episodes to download (missed) and the user should know this.
        if len(rss.new_tracked_and_downloadable) > 0 and notification != rss.new_tracked_and_downloadable:
            Notifier.notify(self._notify_message(rss.new_tracked_and_downloadable),
                            title='TVM - TV Manager',
                            sound='DEFAULT')

            if cache_notification:
                self.cache_notification(rss.new_tracked_and_downloadable)

        # TODO: Email notifications either via (email, smtp) or mailgun.

    def print_updates(self, rss, show_untracked=False):
        info('The following are the new episodes found:')
        for tracked in rss.new_tracked_and_downloadable:
            info2('%s,%s,"%s"' % (tracked['season'], tracked['episode'], tracked['tracked_title']))

        if show_untracked:
            info('-' * 79)
            for untracked in rss.new_untracked_episodes:
                info2(untracked['raw_title'])

        options = tuple(map(len, [rss.new_tracked_and_downloadable, rss.new_untracked_episodes]))
        info('Tracked: %d | Untracked: %d (possible duplicates --untracked).' % options)

    def store_to_cache(self, rss):
        cache_path = self.config.downloads['cache_path']
        if not os.path.exists(cache_path):
            os.makedirs(cache_path)

        now = datetime.now()
        info('Caching cleansed RSS response to: "%s.json"' % os.path.join(cache_path, str(now)))
        with open(os.path.join(cache_path, str(now) + '.json'), 'w') as cached_file:
            json.dump(rss.cleansed_updates, cached_file)

    def load_from_cache(self):
        cache_path = self.config.downloads['cache_path']
        if not os.path.exists(cache_path):
            return []

        cached_objects = []
        for cached_file in os.listdir(cache_path):
            if cached_file.startswith('.'):
                continue

            with open(os.path.join(cache_path, cached_file), 'r') as cached:
                cached_objects.append(json.load(cached))

        return cached_objects

    def get_previous_notification(self):
        notification_path = self.config.last_notification_path
        if not os.path.exists(notification_path):
            return None

        with open(notification_path, 'r') as cached_file:
            return json.load(cached_file)

    def cache_notification(self, notification):
        cache_path = self.config.downloads['cache_path']
        if not os.path.exists(cache_path):
            os.makedirs(cache_path)

        notification_path = self.config.last_notification_path
        info('Caching last notification to: "%s"' % notification_path)
        with open(notification_path, 'w') as cached_file:
            json.dump(notification, cached_file)

    def download_shows(self, rss):
        """ Downloads the tracked_episodes found within the supplied `rss` object. """
        info('Downloading new episodes.')
        for episode in rss.new_tracked_and_downloadable:
            info2('Opening episode link in transmission: %s' % episode['raw_title'])
            open_with_default(episode['link'])
