#!/usr/bin/env python
# -*- coding: utf-8 -*-
from __future__ import print_function

import random
import os
import sys
import tty
import subprocess

from datetime import datetime

from tvm.helpers import base
from tvm.shared.io import err, info, info2
from tvm.shared.io import open_with_default


class PlayHelper(base.BaseHelper):
    def __init__(self, config, collection):
        super(self.__class__, self).__init__(config, collection)

    def _build_episode_path(self, show):
        """ None is returned if unsuccessful. """
        season, episode, title = show
        for path in self.config.physical_storage_paths:
            possible_path = os.path.join(path, '/'.join([title, season]))

            # Get the file extension.
            for root, _, files in os.walk(possible_path):
                for file in files:
                    episode_number = os.path.splitext(file)[0]
                    if episode_number != episode:
                        continue

                    return os.path.join(possible_path, file)
        return None

    def update_records_with(self, episode):
        info('Recording "%s" as viewed.' % ','.join(episode))
        self.collection.save([(datetime.now(), ) + episode], type='viewed')

    def play_episode(self, episode):
        """ Returns True if the episode was successfully played. """
        episode_path = self._build_episode_path(episode)
        if not episode_path:
            return False

        return open_with_default(episode_path)

    def next_to_watch(self, title):
        """ The `title` must be an exact title. @see find_matching_titles to find an exact match. """
        all_episodes, viewed_episodes = map(lambda e: self.collection.sort_episodes(e, desc=False),
                                            self.collection.find_all_episodes(title))

        # We have never seen any episodes yet.
        if not viewed_episodes:
            return all_episodes[0]  # First episode.

        last_viewed_episode = viewed_episodes[-1]

        # The most recently viewed episode is the last episode added.
        if all_episodes[-1] == last_viewed_episode:
            return None

        index = all_episodes.index(last_viewed_episode)
        return all_episodes[index + 1]  # Next episode to watch.

    def undo_last_played(self):
        if not len(self.collection.all_viewed_episodes):  # Nothing has yet to be viewed.
            return None

        # NOTE: Will not work for Windows.
        proc = subprocess.Popen(['tail', '-n-1', self.config.viewed_path], stdout=subprocess.PIPE)
        removed_episode = proc.stdout.read().strip('\n')

        proc = subprocess.Popen(['head', '-n-1', self.config.viewed_path], stdout=subprocess.PIPE)
        updated_file = proc.stdout.read()

        with open(self.config.viewed_path, 'w') as out_file:
            out_file.write(updated_file)

        return removed_episode

    def _create_selection_pool(self, title):
        if title:
            # Return the all episodes as the selection pool if we're up to date with the show.
            recorded, viewed = self.collection.find_all_episodes(title)
            if self.collection.viewed_up_to_date(title):
                return recorded
            return viewed

        # A random selection on all recorded shows that have been viewed.
        return [episode[0] for episode in self.collection.all_viewed_episodes]

    def _get_random_episode(self, selection_pool, previously_randomised):
        if len(selection_pool) == len(previously_randomised):
            return False

        while True:
            episode = selection_pool[random.randint(0, len(selection_pool) - 1)]
            if episode not in previously_randomised:
                previously_randomised.append(episode)
                return episode

    def simulate_random_selection(self, title):
        selection_pool = self._create_selection_pool(title)
        previously_randomised = []

        info('Starting random selection simulation.')

        episode = self._get_random_episode(selection_pool, previously_randomised)
        tty.setcbreak(sys.stdin)
        while True:
            if not episode:
                return info('There are no episodes to randomly select from. Exiting.')

            info2('Play episode "%s" [y/n/q]? ' % ','.join(episode), end='')
            response = sys.stdin.read(1).lower()
            print(response.strip('\n'))  # Display what the user typed after the query.

            if response == 'y':
                info('Opening the episode, "%s".' % ','.join(episode))
                if not self.play_episode(episode):
                    err('Could not open episode "%s".' % ','.join(episode))
                    self.update_records_with(episode)
                return

            elif response == 'n':
                episode = self._get_random_episode(selection_pool, previously_randomised)
            elif response == 'q':
                return info('Leaving random selection simulation.')
