#!/usr/bin/env python
# -*- coding: utf-8 -*-
from __future__ import print_function

import os
import sys
import subprocess


def print_errors(errors, terminate_program=True):
    """ `errors` expected to be in the form:

        {'error_type': [defect, ], }
    """
    # Sort the defects before printing them so it's easier on the eyes.
    for error_type, defects in errors.iteritems():
        if not isinstance(defects, str):
            # Sort by title, season and episode.
            errors[error_type] = sorted(defects, key=lambda d: (d[-1], d[0], d[1]))

            # Convert each list into a string delimited by a comma.
            errors[error_type] = map(lambda d: ','.join(d), errors[error_type])

    error_count = 0
    for error_type, defects in errors.iteritems():
        error_count += len(defects)

        for defect in defects:
            defect = ','.join(defect) if isinstance(defect, list) else defect
            warn('(error) "%s", (defective record) "%s"' % (error_type, defect))

    if error_count > 0:
        warn('There are %d total errors found.' % error_count)
    terminate_program and error_count > 0 and exit(0)


def print_matched_status(matched):
    """ Intended use when the supplied title matches with multiple titles. In all
        cases, we just want a single title.
    """
    if not matched:
        info('No TV shows matching your expression.')
        exit(0)

    if isinstance(matched, list):  # Multiple matches.
        info('Do you mean any of these titles?')
        for match in matched:
            title, ratio, percentage = match
            info2('(%s%%) %s' % (percentage, title))
        exit(0)

    info('Found a match! "%s"' % matched)


def open_with_default(file_path):
    """ Opens a file given the `file_path` using the default application. """
    try:
        if sys.platform == 'darwin':  # Mac OS.
            subprocess.call(['open', file_path])
        elif sys.platform == 'win32':  # Windows 32bit.
            os.startfile(file_path)  # os.startfile only available on Windows.
        elif sys.platform.startswith('linux'):
            subprocess.call(['xdg-open', file_path])
        else:
            err('Unexpected platform "%s". Submit a bug on https://github.com/davidvuong/tvm.' % sys.platform)
            return False

        return True
    except OSError:
        err('Cannot open path because your platform "%s" is not supported.' % sys.platform)

    return False


# Reference: http://pueblo.sourceforge.net/doc/manual/ansi_color_codes.html
def err(error_message):
    print("\033[1;31mERR!\033[0m " + str(error_message), file=sys.stderr)


def warn(warning_message):
    print("\033[33mWARN!\033[0m " + str(warning_message), file=sys.stderr)


def info(info_message, end='\n'):
    print("\033[34m~>\033[0m " + "\033[39m" + str(info_message) + "\033[0m", file=sys.stdout, end=end)


def info2(info_message, end='\n'):
    print("\033[35m⇌   \033[0m " + "\033[39m" + str(info_message) + "\033[0m", file=sys.stdout, end=end)


def ok(ok_message):
    print("\033[32mOK.\033[0m " + str(ok_message), file=sys.stdout)


if __name__ == '__main__':
    err("An error has occurred and something bad is happening.")
    warn("Warnings can be ignored if you want to risk what might happen.")
    info("General status information about what's happening.")
    info2("Some additional information accompanying the above info call.")
    ok("Everything looks good. Good to go!")
