#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""A proxy to online sources for obtaining additional show data. """
import operator
import urllib2
from difflib import SequenceMatcher

from tvm.vendor.tmdb3 import searchSeries
from tvm.vendor.tmdb3 import set_key
from tvm.vendor.tmdb3.tmdb_exceptions import TMDBKeyInvalid

from tvm.shared.io import err
from tvm.shared.dates import date_to_datetime


class DataSource(object):
    def __init__(self, config, query):
        self.config = config
        self._search_result = None

        self._search(query)

    def _search(self, query):
        if not query: return
        try:
            set_key(self.config.api_keys['the_movie_db'])

            # _ delimiter does not play nice with other services.
            result = searchSeries(query.replace('_', ' '))

            if len(result) == 0: return
            if len(result) > 1:
                result = [self._determine_result(query, result)]

            self._search_result = result[0]
        except TMDBKeyInvalid:
            err('the_movie_db api key is invalid.')
        except urllib2.URLError:
            err('Not connected to the internet.')

    def _determine_result(self, query, results):
        ratios = []
        for result in results:
            name = result.name.lower().replace(' ', '_')
            ratios.append((result, SequenceMatcher(None, name, query)))

        return max(ratios, key=operator.itemgetter(1))[0]  # Don't want the ratio.

    @property
    def matched(self):
        """ Returns True if the search found a result. """
        return bool(self._search_result)

    @property
    def id(self):
        return self._search_result.id

    @property
    def title(self):
        return self._search_result.name

    @property
    def alternate_title(self):
        return self._search_result.original_name

    @property
    def in_production(self):
        return self._search_result.in_production

    @property
    def genres(self):
        return [str(g.name) for g in self._search_result.genres]

    @property
    def first_aired(self):
        return date_to_datetime(self._search_result.first_air_date)

    @property
    def last_aired(self):
        return date_to_datetime(self._search_result.last_air_date)

    @property
    def description(self):
        return self._search_result.overview

    @property
    def trailer(self):
        return None  # NOTE: Only available for movies right now.

    @property
    def all(self):
        return {
            'id': self.id,
            'title': self.title,
            'alternate_title': self.alternate_title,
            'in_production': self.in_production,
            'genres': self.genres,
            'first_aired': self.first_aired,
            'last_aired': self.last_aired,
            'description': self.description,
            'trailer': self.trailer
        }
