#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""tvm.py

usage:
    tvm.py generate <path> [--append --force]
    tvm.py doctor [--repository --records]
    tvm.py play (<title>... [--no-open --random]|(--undo|--random))
    tvm.py ls [[<title>... [--expand]]|[--filter=<type>]]
    tvm.py check [[--sleep=<n> --without-cache --untracked]|--download]

commands:
    generate         generates records to be added in shows_added with <path>.
    play             plays the next episode in the given tv show <title>.
    doctor           checks for syntax and semantic errors in the recorded files.
    info             shows basic information about the given tv show <title>.
    ls               lists all tv shows recorded.
    check            checks for new tv shows to download.

generate:
    --append         appends stdout to records.
    --force          pre-tracked records aren't generated, force generates them.

doctor:
    --records        checks if physical files exists in the records.
    --repository     checks if entries in record have a physical counterpart.

play:
    --no-open        records but doesn't open the episode.
    --info           shows basic statistical information.
    --random         randomly selects a show to play.
    --undo           undo the most previously played episode.

ls:
    --expand         completed and pending seasons aren't collapsed.
    --filter=<type>  filters the result based on the filter type.
                        new: filters out shows/episodes without pending episodes.
                        recorded: filters out shows that have not been viewed.
                        nrecorded: opposite of recorded.

check:
    --download       downloads all queued shows as well as checks for new updates.
    --sleep=<n>      sleeps n seconds before performing the check [default: 0].
    --without-cache  checks but does not store results in cache.
    --untracked      shows untracked episodes that can be downloaded.

arguments:
    <path>           directory containing tv shows.
    <title>          title of the tv show.

options:
    -h --help        shows this.
    -v --version     shows version.
"""
from __future__ import print_function

import signal

from os import chdir, path
from docopt import docopt
from inspect import getmembers, isclass

try:
    from commands import *
    from . import __version__
except ImportError:
    import traceback
    traceback.print_exc()

    print('Development mode: Run with -m `python -m tvm/tvm [command] [arguments]`.')
    print('Alternatively, run python setup.py develop; tvm -h')
    exit(0)


def signal_handler(signal, frame):
    exit(0)


signal.signal(signal.SIGINT, signal_handler)


def main():
    arguments = docopt(__doc__, version=__version__)

    # Set the current directory to the directory of this file.
    chdir(path.dirname(path.abspath(__file__)))

    # Dynamically load corresponding command class given command name.
    for command, value in arguments.iteritems():
        if command.startswith('--'):  # Options.
            continue

        if command.startswith('<'):  # Positional arguments.
            continue

        if not value:  # Not a command that was called.
            continue

        _, klass = getmembers(globals()[command], isclass)[0]
        klass(**arguments)

if __name__ == '__main__':
    main()
